==> import_lint-0.8.0.tar.gz: lib/src/import_lint_options.dart <==
import 'dart:convert' as convert;
import 'dart:io' as io;

import 'package:glob/glob.dart';
import 'package:yaml/yaml.dart' as yaml;

class ImportLintOptions {
  const ImportLintOptions({required this.rules, required this.common});

  factory ImportLintOptions.init({
    required String directoryPath,
    required String optionsFilePath,
  }) {
    final rules = RulesOption.fromOptionsFile(optionsFilePath);
    final common = CommonOption.fromYaml(directoryPath);
    return ImportLintOptions(
      rules: rules,
      common: common,
    );
  }

  final RulesOption rules;
  final CommonOption common;
}

class CommonOption {
  const CommonOption({
    required this.directoryPath,
    required this.packageName,
  });

  factory CommonOption.fromYaml(String directoryPath) {
    return CommonOption(
      directoryPath: directoryPath,
      packageName: _packageName(directoryPath),
    );
  }

  final String directoryPath;
  final String packageName;

  static String _packageName(String directoryPath) {
    final pubspecFile = io.File('$directoryPath/$_pubspecFileName');
    late String value;
    try {
      value = pubspecFile.readAsStringSync();
    } on Exception catch (e) {
      throw Exception(
        'Not found pubspec.yaml file'
        'at the root of your project.'
        '\n$e',
      );
    }
    final loadYaml = yaml.loadYaml(value);
    return loadYaml['name'];
  }

  static const _pubspecFileName = 'pubspec.yaml';
}

class RulesOption {
  const RulesOption(this.value);
  factory RulesOption.fromOptionsFile(String path) {
    late String readValue;
    try {
      readValue = io.File(path).readAsStringSync();
    } on Exception catch (e) {
      throw Exception(
        'Not found analysis_options.yaml file '
        'at the root of your project.'
        '\n $e',
      );
    }
    final loadedYaml = yaml.loadYaml(readValue);
    final encoded = convert.jsonEncode(loadedYaml['import_lint']['rules']);
    final rulesMap = convert.jsonDecode(encoded) as Map<String, dynamic>;

    final ruleNames = rulesMap.keys.toList();

    final result = <Rule>[];
    for (final name in ruleNames) {
      final ruleMap = rulesMap[name];
      final rule = Rule.ofMap(ruleMap: ruleMap, name: name);

      result.add(rule);
    }
    return RulesOption(result);
  }
  final List<Rule> value;
}

class Rule {
  const Rule({
    required this.name,
    required this.targetFilePath,
    required this.notAllowImports,
    required this.excludeImports,
  });

  factory Rule.ofMap({
    required Map<String, dynamic> ruleMap,
    required String name,
  }) {
    final targetFilePath = Glob(ruleMap['target_file_path'],
        recursive: true, caseSensitive: false);

    final notAllowImports = (ruleMap['not_allow_imports'] as List<dynamic>)
        .map((e) => Glob(e.toString(), recursive: true, caseSensitive: false))
        .toList();
    final excludeImports = (ruleMap['exclude_imports'] as List<dynamic>)
        .map((e) => Glob(e.toString(), recursive: true, caseSensitive: false))
        .toList();

    return Rule(
      name: name,
      targetFilePath: targetFilePath,
      notAllowImports: notAllowImports,
      excludeImports: excludeImports,
    );
  }

  final String name;
  final Glob targetFilePath;
  final List<Glob> notAllowImports;
  final List<Glob> excludeImports;
}

==> import_lint-0.9.1.tar.gz: lib/src/import_lint_options.dart <==
import 'dart:convert' as convert;
import 'dart:io' as io;

import 'package:glob/glob.dart';
import 'package:yaml/yaml.dart' as yaml;

class ImportLintOptions {
  const ImportLintOptions({required this.rules, required this.common});

  factory ImportLintOptions.init({
    required String directoryPath,
    required String optionsFilePath,
  }) {
    final common = CommonOption.fromYaml(directoryPath);
    final rules = RulesOption.fromOptionsFile(optionsFilePath, common);
    return ImportLintOptions(
      rules: rules,
      common: common,
    );
  }

  final RulesOption rules;
  final CommonOption common;
}

class CommonOption {
  const CommonOption({
    required this.directoryPath,
    required this.packageName,
  });

  factory CommonOption.fromYaml(String directoryPath) {
    return CommonOption(
      directoryPath: directoryPath,
      packageName: _packageName(directoryPath),
    );
  }

  final String directoryPath;
  final String packageName;

  static String _packageName(String directoryPath) {
    final pubspecFile = io.File('$directoryPath/$_pubspecFileName');
    late String value;
    try {
      value = pubspecFile.readAsStringSync();
    } on Exception catch (e) {
      throw Exception(
        'Not found pubspec.yaml file'
        'at the root of your project.'
        '\n$e',
      );
    }
    final loadYaml = yaml.loadYaml(value);
    return loadYaml['name'];
  }

  static const _pubspecFileName = 'pubspec.yaml';
}

class RulesOption {
  const RulesOption(this.value);
  factory RulesOption.fromOptionsFile(String path, CommonOption commonOption) {
    late String readValue;
    try {
      readValue = io.File(path).readAsStringSync();
    } on Exception catch (e) {
      throw Exception(
        'Not found analysis_options.yaml file '
        'at the root of your project.'
        '\n $e',
      );
    }
    final loadedYaml = yaml.loadYaml(readValue);
    final encoded = convert.jsonEncode(loadedYaml['import_lint']['rules']);
    final rulesMap = convert.jsonDecode(encoded) as Map<String, dynamic>;

    final ruleNames = rulesMap.keys.toList();

    final result = <Rule>[];

    for (final name in ruleNames) {
      final ruleMap = rulesMap[name];
      final rule =
          Rule.ofMap(ruleMap: ruleMap, name: name, commonOption: commonOption);

      result.add(rule);
    }
    return RulesOption(result);
  }
  final List<Rule> value;
}

class Rule {
  const Rule({
    required this.name,
    required this.targetFilePath,
    required this.notAllowImports,
    required this.excludeImports,
  });

  factory Rule.ofMap({
    required Map<String, dynamic> ruleMap,
    required String name,
    required CommonOption commonOption,
  }) {
    final targetFilePath = Glob(ruleMap['target_file_path'],
        recursive: true, caseSensitive: false);

    final notAllowImports = (ruleMap['not_allow_imports'] as List<dynamic>)
        .map((e) => ImportRulePath.from(e.toString(), commonOption))
        .toList();
    final excludeImports = (ruleMap['exclude_imports'] as List<dynamic>)
        .map((e) => ImportRulePath.from(e.toString(), commonOption))
        .toList();

    return Rule(
      name: name,
      targetFilePath: targetFilePath,
      notAllowImports: notAllowImports,
      excludeImports: excludeImports,
    );
  }

  final String name;
  final Glob targetFilePath;
  final List<ImportRulePath> notAllowImports;
  final List<ImportRulePath> excludeImports;
}

class ImportRulePath {
  const ImportRulePath(this.package, this.path);

  factory ImportRulePath.from(String value, CommonOption commonOption) {
    final package = RegExp('(?<=package:).*?(?=\/)').stringMatch(value);
    if (package != null) {
      final importPath = value.replaceFirst('package:$package/', '');
      final path = Glob(importPath, recursive: true, caseSensitive: false);

      return ImportRulePath(package, path);
    }

    final path = Glob(value, recursive: true, caseSensitive: false);
    return ImportRulePath(commonOption.packageName, path);
  }

  final String package;
  final Glob path;
}
